package inf101.v18.rogue101.objects;

import inf101.v18.grid.GridDirection;
import inf101.v18.rogue101.game.IGame;
import javafx.scene.input.KeyCode;
import java.util.ArrayList;
import java.util.List;

public class Player implements IPlayer {
    private int hp = getMaxHealth();
    private List<IItem> equipped = new ArrayList<>();

    @Override
    public void keyPressed(IGame game, KeyCode key) {
        if (key == KeyCode.LEFT || key == KeyCode.A) {
            tryToMove(game, GridDirection.WEST);
        } else if (key == KeyCode.RIGHT || key == KeyCode.D) {
            tryToMove(game, GridDirection.EAST);
        } else if (key == KeyCode.UP || key == KeyCode.W) {
            tryToMove(game, GridDirection.NORTH);
        } else if (key == KeyCode.DOWN || key == KeyCode.S) {
            tryToMove(game, GridDirection.SOUTH);
        } else if (key == KeyCode.E) {
            pickUp(game);
        } else if (key == KeyCode.Q) {
            drop(game);
        }
        showStatus(game);
    }

    private void pickUp(IGame game) {
        //TODO: If there are several items on a tile, let player choose what to pick up.
        if (equipped.size() < 5) {
            List<IItem> items = game.getLocalItems();
            for (IItem item : items) {
                IItem pickedUp = game.pickUp(item);
                if (pickedUp != null) {
                    equipped.add(pickedUp);
                    game.displayMessage("Picked up " + pickedUp.getName());
                    return;
                }
            }
            game.displayMessage("There is nothing to pick up.");
        } else {
            game.displayMessage("Your inventory is full.");
        }
    }

    private void pickUpV2(IGame game) {
        List<IItem> items = game.getLocalItems();
        if (items.size() < 1) {
            game.displayMessage("There is nothing to pick up.");
            return;
        }
        StringBuilder msg = new StringBuilder("Items on this tile:");
        for (int i = 0; i < items.size(); i++) {
            msg.append(" [").append(i).append("] ").append(items.get(i).getName());
        }
        game.displayMessage(msg.toString());
        if (equipped.size() < 5) {
            //TODO: Let user choose 1-5.
        } else {
            game.displayMessage("Your inventory is full.");
        }
    }

    private void drop(IGame game) {
        //TODO: Find a way to implement V2.
        if (!equipped.isEmpty()) {
            if (game.drop(equipped.get(0))) {
                equipped.remove(0);
                game.displayMessage("Item dropped.");
            } else {
                game.displayMessage("The ground rejected the item.");
            }
        } else {
            game.displayMessage("You have nothing to drop.");
        }
    }

    private void showStatus(IGame game) {
        List<String> items = new ArrayList<>();
        for (IItem item : equipped) {
            String name = item.getName();
            items.add(Character.toUpperCase(name.charAt(0)) + name.substring(1));
        }
        game.formatStatus("HP: %d/%d ATK: %d DEF: %s INV: %s", hp, getMaxHealth(), getAttack(), getDefence(), String.join(",", items));
    }

    private void tryToMove(IGame game, GridDirection dir) {
        if (game.canGo(dir)) {
            game.move(dir);
        } else {
           game.displayMessage("Umm, it is not possible to move in that direction.");
        }
    }

    @Override
    public int getAttack() {
        return 10;
    }

    @Override
    public int getDamage() {
        return 10;
    }

    @Override
    public int getCurrentHealth() {
        return hp;
    }

    @Override
    public int getDefence() {
        return 1;
    }

    @Override
    public int getMaxHealth() {
        return 100;
    }

    @Override
    public String getName() {
        return "Person";
    }

    @Override
    public int getSize() {
        return 10;
    }

    @Override
    public String getSymbol() {
        return "X";
    }

    @Override
    public int handleDamage(IGame game, IItem source, int amount) {
        hp -= amount;
        return amount;
    }
}
