package inf101.v18.rogue101.tests;

import static org.junit.Assert.*;

import org.junit.Test;

import inf101.v18.grid.GridDirection;
import inf101.v18.grid.ILocation;
import inf101.v18.rogue101.game.Game;
import inf101.v18.rogue101.objects.IPlayer;
import javafx.scene.input.KeyCode;

public class PlayerTest {
	//Tiny maps in collaboration with Stian J. Husum
	private static String NO_WALLS_MAP = "1 1\n"
			+ "@\n";

	private static String EMPTY_MAP = "5 5\n"
			+ "#####\n"
			+ "#   #\n"
			+ "# @ #\n"
			+ "#   #\n"
			+ "#####\n";

	private static String RABBIT_MAP = "5 5\n"
			+ "#####\n"
			+ "#RRR#\n"
			+ "#R@R#\n"
			+ "#RRR#\n"
			+ "#####\n";

	private static String CARROT_MAP = "5 5\n"
			+ "#####\n"
			+ "#CCC#\n"
			+ "#C@C#\n"
			+ "#CCC#\n"
			+ "#####\n";

	@Test
	public void testOutOfBounds() {
		Game game = new Game(NO_WALLS_MAP);
		IPlayer player = (IPlayer) game.setCurrent(0, 0);
		ILocation loc = game.getLocation();
		player.keyPressed(game, KeyCode.LEFT);
		assertEquals(loc, game.getLocation());
	}

	@Test
	public void testActor() {
		Game game = new Game(RABBIT_MAP);
		IPlayer player = (IPlayer) game.setCurrent(2, 2);
		ILocation loc = game.getLocation();
		player.keyPressed(game, KeyCode.LEFT);
		assertEquals(loc, game.getLocation());
		player.keyPressed(game, KeyCode.RIGHT);
		assertEquals(loc, game.getLocation());
		player.keyPressed(game, KeyCode.UP);
		assertEquals(loc, game.getLocation());
		player.keyPressed(game, KeyCode.DOWN);
		assertEquals(loc, game.getLocation());
	}

	@Test
	public void testItem() {
		Game game = new Game(CARROT_MAP);
		IPlayer player = (IPlayer) game.setCurrent(2, 2);
		ILocation loc = game.getLocation();
		player.keyPressed(game, KeyCode.RIGHT);
		assertEquals(loc.go(GridDirection.EAST), game.getLocation());
		game.doTurn();
		player.keyPressed(game, KeyCode.LEFT);
		game.doTurn();
		player.keyPressed(game, KeyCode.LEFT);
		assertEquals(loc.go(GridDirection.WEST), game.getLocation());
		game.doTurn();
		player.keyPressed(game, KeyCode.UP);
		game.doTurn();
		player.keyPressed(game, KeyCode.RIGHT);
		assertEquals(loc.go(GridDirection.NORTH), game.getLocation());
		game.doTurn();
		player.keyPressed(game, KeyCode.DOWN);
		game.doTurn();
		player.keyPressed(game, KeyCode.DOWN);
		assertEquals(loc.go(GridDirection.SOUTH), game.getLocation());
	}

	@Test
	public void testWallsAndKeys() {
		Game game = new Game(EMPTY_MAP);
		IPlayer player = (IPlayer) game.setCurrent(2, 2);
		ILocation loc = game.getLocation();
		player.keyPressed(game, KeyCode.UP);
		game.doTurn();
		player.keyPressed(game, KeyCode.UP);
		game.doTurn();
		assertEquals(loc.go(GridDirection.NORTH), game.getLocation());
		player.keyPressed(game, KeyCode.DOWN);
		game.doTurn();
		player.keyPressed(game, KeyCode.RIGHT);
		game.doTurn();
		player.keyPressed(game, KeyCode.RIGHT);
		game.doTurn();
		assertEquals(loc.go(GridDirection.EAST), game.getLocation());
		player.keyPressed(game, KeyCode.LEFT);
		game.doTurn();
		player.keyPressed(game, KeyCode.DOWN);
		game.doTurn();
		player.keyPressed(game, KeyCode.DOWN);
		game.doTurn();
		assertEquals(loc.go(GridDirection.SOUTH), game.getLocation());
		player.keyPressed(game, KeyCode.UP);
		game.doTurn();
		player.keyPressed(game, KeyCode.LEFT);
		game.doTurn();
		player.keyPressed(game, KeyCode.LEFT);
		assertEquals(loc.go(GridDirection.WEST), game.getLocation());
	}
}
